function rotated_image =  autorotate(filename, extension)
% implementation of the autorotate algorithm given by Bieniecki et al.
% give this function the name of the file output by hasankaram.m
% (we're rotating the text region selected by that algorithm)

% read the output from the text detection - should be grayscale with
% background white (255) and foreground black (0)
orig = imread([filename extension]);
[rows_image cols_image] = size(orig);

% find the slope for every region in this image
load([filename '_all_regions.mat']);
all_regions = unique(final_region_image(find(final_region_image ~= 0)))

rotation_angles = [];

for region_index = 1:size(all_regions, 1)
orig_region = all_regions(region_index)

%show us the region 
potential_rows = mod(find(final_region_image == orig_region), rows_image);
potential_rows(potential_rows == 0) = rows_image;
top_row = min(potential_rows);
potential_cols = ceil(find(final_region_image == orig_region) / rows_image);
left_col = min(potential_cols);
right_col = max(potential_cols);

left_bottom_col = left_col;
left_bottom_row = find(final_region_image(:, left_bottom_col) == orig_region);

%find all baseline points through this component - hopefully determining
%a skeleton of the line's skew

curr_point = 1;
basepoints = [];

for curr_col = left_col:2:right_col

%find the pixels along this column belonging to the original region
all_rows = find(final_region_image(:, curr_col) == orig_region);
if size(all_rows, 1) == 0 || size(all_rows, 2) == 0
continue
end

%find the baseline point and store it
base_row = max(all_rows);
basepoints(curr_point, :) = [base_row, curr_col];
curr_point = curr_point + 1;

%end for curr_col = left_col:2:right_col
end

%figure out the mean difference and standard deviation from it
differences = [basepoints(2:curr_point - 1, 1); 0] - basepoints(:,1);
differences = differences / 2; %makes this average difference along slope
differences(curr_point - 1,:) = [];
mean_diff = mean(differences);
std_diff = std(differences);

%remove all points too far from it
curr_mid = 2;
  
for index = 2:curr_point - 1
if (basepoints(curr_mid, 1) - basepoints(curr_mid - 1, 1)) / (basepoints(curr_mid, 2) - basepoints(curr_mid - 1, 2))  > mean_diff + std_diff || (basepoints(curr_mid, 1) - basepoints(curr_mid - 1, 1)) / (basepoints(curr_mid, 2) - basepoints(curr_mid - 1, 2)) < mean_diff - std_diff
basepoints(curr_mid, :) = [];
else
curr_mid = curr_mid + 1;
end
end

%trim the leading and trailing datapoints for good measure - get
%rid of initial/final letter curves
[num_entries dont_care] = size(basepoints);
if num_entries > 5
basepoints(1,:) = [];
basepoints(1,:) = [];
basepoints(num_entries - 3,:) = [];
basepoints(num_entries - 3,:) = [];
end

%fit the data in basepoints to a line and figure out this line's slope
if size(basepoints, 1) >= 2
my_fit = fittype({'x', '1'});
my_linear_model = fit(basepoints(:,2), basepoints(:,1), my_fit);
my_coefficients = coeffvalues(my_linear_model);
slope = my_coefficients(1);

%figure out rotation angle based on slope
rotation_angles(region_index) = atand(slope)
else
rotation_angles(region_index) = 0
end

%end for region_index = 1:size(all_regions, 1)
end

%remove any zeros
new_angles = [];
new_index = 1;
for old_index = 1:size(rotation_angles, 1) * size(rotation_angles, 2)
if abs(rotation_angles(old_index)) > 0.000001
new_angles(new_index) = rotation_angles(old_index);
new_index = new_index + 1;
end
end

rotation_angles = new_angles;

%eliminate any angle outside 3/4 SD from the mean
mean_angle = mean(rotation_angles)
std_angle = std(rotation_angles)
new_angles = [];
new_index = 1;
for old_index = 1:size(rotation_angles, 1) * size(rotation_angles, 2)
if rotation_angles(old_index) <= mean_angle + 0.75 * std_angle && rotation_angles(old_index) >= mean_angle - 0.75 * std_angle
new_angles(new_index) = rotation_angles(old_index);
new_index = new_index + 1;
end
end

rotation_angles = new_angles

%the rotation angle will now be the mean angle - hopefully a more
%balanced representation of the angle we actually want
if size(rotation_angles, 1) > 0 && size(rotation_angles, 2) > 0
rotation_angle = mean(rotation_angles)

%center the image around the left corner
centered_image = zeros(rows_image * 2 - 1, cols_image * 2 - 1, 'uint8');
centered_image(:,:) = 255;
center_row = rows_image; % new row coordinate for left bottom corner
center_col = cols_image; % new col coordinate for left bottom corner

centered_image(center_row - left_bottom_row : center_row - left_bottom_row + rows_image - 1, center_col - left_bottom_col : center_col - left_bottom_col  + cols_image - 1) = orig;
centered_image(centered_image == 1) = 255;
centered_image(centered_image == 0) = 128;

%rotate the image <rotation_angle> degrees counterclockwise
%around the bottom line's left lower corner
rotated_image = imrotate(centered_image, rotation_angle);
[rotated_rows rotated_cols] = size(rotated_image);

%after the rotation, clean up background and text so final text
%is black, everything else white
rotated_image(rotated_image == 0) = 255;
rotated_image(rotated_image == 128) = 0;

%crop to smallest possible region containing text, plus a few px padding
matching_rows = mod(find(rotated_image == 0), rotated_rows);
matching_rows(matching_rows == 0) = rotated_rows;
min_row = min(matching_rows);
max_row = max(matching_rows);
matching_cols = ceil(find(rotated_image == 0) / rotated_rows);
min_col = min(matching_cols);
max_col = max(matching_cols);

top_wiggle = min_row;
bottom_wiggle = rotated_rows - max_row;
left_wiggle = min_col;
right_wiggle = rotated_cols - max_col;

rotated_image = rotated_image(max(min_row - 3, min_row - top_wiggle + 1):min(max_row + 3, max_row + bottom_wiggle), max(min_col - 3, min_col - left_wiggle + 1):min(max_col + 3, max_col + right_wiggle));

imwrite(rotated_image, [filename '_rotated.tif'], 'tif', 'Compression', 'none');

%if for some reason we didn't have any rotation angles left, don't rotate
else
imwrite(orig, [filename '_rotated.tif'], 'tif', 'Compression', 'none');
end
